'''
 # @ Author: Keivan Tafakkori
 # @ Created: 2023-05-11
 # @ Modified: 2023-05-12
 # @ Contact: https://www.linkedin.com/in/keivan-tafakkori/
 # @ Github: https://github.com/ktafakkori
 # @ Website: https://ktafakkori.github.io/
 # @ Copyright: 2023. MIT License. All Rights Reserved.
 '''


from .feloopy import *
from .operators import *
from .algorithms.mcdm import *

HEURISTIC_ALGORITHMS = [

    #MealPy
    ['mealpy', 'orig-ep'], 
    ['mealpy', 'levy-ep'], 
    ['mealpy', 'orig-es'], 
    ['mealpy', 'levy-es'], 
    ['mealpy', 'orig-ma'], 
    ['mealpy', 'base-ga'], 
    ['mealpy', 'single-ga'], 
    ['mealpy', 'multi-ga'], 
    ['mealpy', 'elite-single-ga'], 
    ['mealpy', 'elite-multi-ga'], 
    ['mealpy', 'base-de'],
    ['mealpy', 'ja-de'], 
    ['mealpy', 'sa-de'], 
    ['mealpy', 'sha-de'], 
    ['mealpy', 'l-sha-de'], 
    ['mealpy', 'sap-de'], 
    ['mealpy', 'orig-fpa'], 
    ['mealpy', 'orig-cro'], 
    ['mealpy', 'o-cro'], 
    ['mealpy', 'cma-es'], 
    ['mealpy', 'simp-cma-es'], 
    ['mealpy', 'orig-pso'], 
    ['mealpy', 'p-pso'], 
    ['mealpy', 'h-pso-tvac'], 
    ['mealpy', 'c-pso'], 
    ['mealpy', 'cl-pso'], 
    ['mealpy', 'orig-bfo'], 
    ['mealpy', 'orig-beesa'], 
    ['mealpy', 'a-bfo'], 
    ['mealpy', 'prob-beesa'], 
    ['mealpy', 'orig-cso'], 
    ['mealpy', 'orig-abc'], 
    ['mealpy', 'orig-acor'], 
    ['mealpy', 'orig-csa'], 
    ['mealpy', 'orig-ffa'], 
    ['mealpy', 'orig-fa'], 
    ['mealpy', 'orig-ba'], 
    ['mealpy', 'adap-ba'], 
    ['mealpy', 'modi-ba'], 
    ['mealpy', 'orig-foa'], 
    ['mealpy', 'base-foa'], 
    ['mealpy', 'whale-foa'], 
    ['mealpy', 'orig-sspidero'], 
    ['mealpy', 'orig-gwo'], 
    ['mealpy', 'rw-gwo'], 
    ['mealpy', 'orig-sspidera'], 
    ['mealpy', 'orig-alo'], 
    ['mealpy', 'base-alo'], 
    ['mealpy', 'orig-mfo'], 
    ['mealpy', 'base-mfo'], 
    ['mealpy', 'orig-eho'], 
    ['mealpy', 'orig-ja'], 
    ['mealpy', 'base-ja'], 
    ['mealpy', 'levy-ja'], 
    ['mealpy', 'orig-woa'], 
    ['mealpy', 'hi-woa'], 
    ['mealpy', 'orig-do'], 
    ['mealpy', 'orig-bsa'], 
    ['mealpy', 'orig-sho'], 
    ['mealpy', 'orig-sso'], 
    ['mealpy', 'orig-srsr'], 
    ['mealpy', 'orig-goa'], 
    ['mealpy', 'orig-coa'], 
    ['mealpy', 'orig-msa'], 
    ['mealpy', 'orig-slo'], 
    ['mealpy', 'modi-slo'], 
    ['mealpy', 'impr-slo'], 
    ['mealpy', 'orig-nmra'], 
    ['mealpy', 'impr-nmra'], 
    ['mealpy', 'orig-pfa'], 
    ['mealpy', 'orig-sfo'], 
    ['mealpy', 'impr-sfo'], 
    ['mealpy', 'orig-hho'], 
    ['mealpy', 'orig-mrfo'], 
    ['mealpy', 'orig-bes'], 
    ['mealpy', 'orig-ssa'], 
    ['mealpy', 'base-ssa'], 
    ['mealpy', 'orig-hgs'], 
    ['mealpy', 'orig-ao'], 
    ['mealpy', 'gwo-woa'], 
    ['mealpy', 'orig-mpa'], 
    ['mealpy', 'orig-hba'], 
    ['mealpy', 'orig-scso'], 
    ['mealpy', 'orig-tso'], 
    ['mealpy', 'orig-avoa'], 
    ['mealpy', 'orig-agto'], 
    ['mealpy', 'orig-aro'], 
    ['mealpy', 'levy-aro'], 
    ['mealpy', 'selec-aro'], 
    ['mealpy', 'wmqi-mrfo'], 
    ['mealpy', 'orig-esoa'], 
    ['mealpy', 'sea-ho'], 
    ['mealpy', 'orig-mgo'], 
    ['mealpy', 'orig-gjo'], 
    ['mealpy', 'orig-fox'], 
    ['mealpy', 'orig-gto'],
    ['mealpy', 'modi101-gto'], 
    ['mealpy', 'modi102-gto'], 
    ['mealpy', 'orig-sa'], 
    ['mealpy', 'orig-wdo'], 
    ['mealpy', 'orig-mvo'], 
    ['mealpy', 'base-mvo'], 
    ['mealpy', 'orig-two'], 
    ['mealpy', 'oppo-two'], 
    ['mealpy', 'levy-two'], 
    ['mealpy', 'enha-two'], 
    ['mealpy', 'orig-efo'], 
    ['mealpy', 'base-efo'], 
    ['mealpy', 'orig-nro'], 
    ['mealpy', 'orig-hgso'], 
    ['mealpy', 'orig-aso'], 
    ['mealpy', 'orig-eo'], 
    ['mealpy', 'modi-eo'], 
    ['mealpy', 'adap-eo'], 
    ['mealpy', 'orig-archoa'], 
    ['mealpy', 'orig-rime'], 
    ['mealpy', 'orig-evo'], 
    ['mealpy', 'orig-cdo'], 
    ['mealpy', 'orig-fla'], 
    ['mealpy', 'orig-ca'], 
    ['mealpy', 'orig-ica'], 
    ['mealpy', 'orig-tlo'], 
    ['mealpy', 'base-tlo'], 
    ['mealpy', 'itlo'], 
    ['mealpy', 'orig-bso'], 
    ['mealpy', 'impr-bso'], 
    ['mealpy', 'orig-qsa'], 
    ['mealpy', 'base-qsa'], 
    ['mealpy', 'oppo-qsa'], 
    ['mealpy', 'levy-qsa'], 
    ['mealpy', 'impr-qsa'], 
    ['mealpy', 'orig-saro'],
    ['mealpy', 'base-saro'], 
    ['mealpy', 'orig-lco'], 
    ['mealpy', 'base-lco'], 
    ['mealpy', 'impr-lco'], 
    ['mealpy', 'orig-ssdo'], 
    ['mealpy', 'orig-gska'], 
    ['mealpy', 'base-gska'], 
    ['mealpy', 'orig-chio'], 
    ['mealpy', 'base-chio'], 
    ['mealpy', 'orig-fbio'], 
    ['mealpy', 'base-fbio'], 
    ['mealpy', 'orig-bro'], 
    ['mealpy', 'base-bro'], 
    ['mealpy', 'orig-spbo'], 
    ['mealpy', 'dev-spbo'], 
    ['mealpy', 'orig-dmoa'], 
    ['mealpy', 'dev-dmoa'], 
    ['mealpy', 'orig-huco'], 
    ['mealpy', 'orig-warso'], 
    ['mealpy', 'orig-hbo'], 
    ['mealpy', 'orig-iwo'], 
    ['mealpy', 'orig-bboa'], 
    ['mealpy', 'base-bbo'], 
    ['mealpy', 'orig-vcs'], 
    ['mealpy', 'base-vcs'], 
    ['mealpy', 'orig-sbo'], 
    ['mealpy', 'base-sbo'], 
    ['mealpy', 'orig-eoa'], 
    ['mealpy', 'orig-who'], 
    ['mealpy', 'orig-sma'], 
    ['mealpy', 'base-sma'], 
    ['mealpy', 'orig-bmo'], 
    ['mealpy', 'orig-tsa'], 
    ['mealpy', 'orig-sos'], 
    ['mealpy', 'orig-soa'], 
    ['mealpy', 'dev-soa'], 
    ['mealpy', 'orig-gco'], 
    ['mealpy', 'base-gco'], 
    ['mealpy', 'orig-wca'], 
    ['mealpy', 'orig-aeo'], 
    ['mealpy', 'enha-aeo'], 
    ['mealpy', 'modi-aeo'], 
    ['mealpy', 'impr-aeo'], 
    ['mealpy', 'augm-aeo'], 
    ['mealpy', 'orig-hc'], 
    ['mealpy', 'swarm-hc'], 
    ['mealpy', 'orig-cem'], 
    ['mealpy', 'orig-sca'], 
    ['mealpy', 'base-sca'], 
    ['mealpy', 'orig-beesa'], 
    ['mealpy', 'orig-cgo'], 
    ['mealpy', 'orig-gbo'], 
    ['mealpy', 'orig-info'], 
    ['mealpy', 'orig-pss'], 
    ['mealpy', 'orig-run'], 
    ['mealpy', 'orig-circle-sa'], 
    ['mealpy', 'ql-sca'], 
    ['mealpy', 'orig-shio'], 
    ['mealpy', 'orig-hs'], 
    ['mealpy', 'base-hs'],
    
    #FelooPy
    ['feloopy', 'ga'],
    ['feloopy', 'sa'],
    ['feloopy', 'ts'],
    ['feloopy', 'de'],
    ['feloopy', 'gwo'],
    ]

EXACT_ALGORITHMS = [

    # gekko
    ['gekko', 'apopt'],
    ['gekko', 'bpopt'],
    ['gekko', 'ipopt'],

    # ortools
    ['ortools', 'bop'],
    ['ortools', 'cbc'],
    ['ortools', 'clp'],
    ['ortools', 'cplex-'],
    ['ortools', 'cplex'],
    ['ortools', 'glop'],
    ['ortools', 'glpk-'],
    ['ortools', 'glpk'],
    ['ortools', 'gurobi-'],
    ['ortools', 'gurobi'],
    ['ortools', 'sat'],
    ['ortools', 'scip'],
    ['ortools', 'xpress-'],
    ['ortools', 'xpress'],

    # pulp
    ['pulp', 'cbc'],
    ['pulp', 'choco'],
    ['pulp', 'coin'],
    ['pulp', 'coinmp-dll'],
    ['pulp', 'cplex-py'],
    ['pulp', 'cplex'],
    ['pulp', 'glpk'],
    ['pulp', 'gurobi-cmd'],
    ['pulp', 'gurobi'],
    ['pulp', 'highs'],
    ['pulp', 'mipcl'],
    ['pulp', 'mosek'],
    ['pulp', 'pyglpk'],
    ['pulp', 'scip'],
    ['pulp', 'xpress-py'],
    ['pulp', 'xpress'],

    # cvxpy
    ['cvxpy', 'osqp'],
    ['cvxpy', 'ecos'],
    ['cvxpy', 'cvxopt'],
    ['cvxpy', 'scs'],
    ['cvxpy', 'scipy'],
    ['cvxpy', 'glop'],
    ['cvxpy', 'glpk'],
    ['cvxpy', 'glpk-mi'],
    ['cvxpy', 'gurobi'],
    ['cvxpy', 'mosek'],
    ['cvxpy', 'cbc'],
    ['cvxpy', 'cplex'],
    ['cvxpy', 'nag'],
    ['cvxpy', 'pdlp'],
    ['cvxpy', 'scip'],
    ['cvxpy', 'xpress'],
    ['cvxpy', 'copt'],
    ['cvxpy', 'clarabel'],
    ['cvxpy', 'proxqp'],

    # cylp
    ['cylp', 'cbc'],

    # gurobi
    ['gurobi', 'gurobi'],

    # cplex
    ['cplex', 'cplex'],

    # linopy
    ['linopy', 'cbc'],
    ['linopy', 'glpk'],
    ['linopy', 'highs'],
    ['linopy', 'gurobi'],
    ['linopy', 'xpress'],
    ['linopy', 'cplex'],

    # mip
    ['mip', 'cbc'],

    # picos
    ['picos', 'cplex'],
    ['picos', 'cvxopt'],
    ['picos', 'ecos'],
    ['picos', 'glpk'],
    ['picos', 'gurobi'],
    ['picos', 'mosek'],
    ['picos', 'mskfn'],
    ['picos', 'osqp'],
    ['picos', 'scip'],
    ['picos', 'smcp'],

    # pymprog
    ['pymprog', 'glpk'],

    # pyomo
    ['pyomo', 'baron'],
    ['pyomo', 'cbc'],
    ['pyomo', 'conopt'],
    ['pyomo', 'cplex'],
    ['pyomo', 'cplex-direct'],
    ['pyomo', 'cplex-persistent'],
    ['pyomo', 'cyipopt'],
    ['pyomo', 'gams'],
    ['pyomo', 'highs'],
    ['pyomo', 'asl'],
    ['pyomo', 'gdpopt'],
    ['pyomo', 'gdpopt.gloa'],
    ['pyomo', 'gdpopt.lbb'],
    ['pyomo', 'gdpopt.loa'],
    ['pyomo', 'gdpopt.ric'],
    ['pyomo', 'glpk'],
    ['pyomo', 'gurobi'],
    ['pyomo', 'gurobi-direct'],
    ['pyomo', 'gurobi-persistent'],
    ['pyomo', 'ipopt'],
    ['pyomo', 'mindtpy'],
    ['pyomo', 'mosek'],
    ['pyomo', 'mosek-direct'],
    ['pyomo', 'mosek-persistent'],
    ['pyomo', 'mpec-minlp'],
    ['pyomo', 'mpec-nlp'],
    ['pyomo', 'multistart'],
    ['pyomo', 'path'],
    ['pyomo', 'scip'],
    ['pyomo', 'trustregion'],
    ['pyomo', 'xpress'],
    ['pyomo', 'xpress-direct'],
    ['pyomo', 'xpress-persistent'],

    # xpress
    ['xpress', 'xpress'],
]

CONSTRAINT_ALGORITHMS = [
    ['cplex-cp', 'cplex'],
    ['ortools-cp', 'ortools']
]

MOO_ALGORITHMS = [

    ['pymultiobjective', 'ns-ga-ii'],
    ['pymultiobjective', 'cta-ea'],
    ['pymultiobjective', 'gr-ea'],
    ['pymultiobjective', 'est-hv'],
    ['pymultiobjective', 'ea-fc'],
    ['pymultiobjective', 'ea-hv'],
    ['pymultiobjective', 'ea-d'],
    ['pymultiobjective', 'na-ea'],
    ['pymultiobjective', 'ns-ga-ii'],
    ['pymultiobjective', 'ns-ga-iii'],
    ['pymultiobjective', 'modi-pso'],
    ['pymultiobjective', 'pa-es'],
    ['pymultiobjective', 'rv-ea'],
    ['pymultiobjective', 'sm-pso'],
    ['pymultiobjective', 'sms-ea'],
    ['pymultiobjective', 'sp-ea-ii'],
    ['pymultiobjective', 'u-ns-ga-iii']

]
